﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO.IsolatedStorage;
using $safeprojectname$;
using System.Collections.ObjectModel;

namespace $safeprojectname$.Model
{
    public class UserSettings
    {
        IsolatedStorageSettings settings;
        public event EventHandler SettingsChanged;

        const string SettingBooleanKeyName = "SettingBoolean";
        const bool SettingBooleanDefault = true;
        public bool SettingBoolean
        {
            get
            {
                return GetValueOrDefault<bool>(SettingBooleanKeyName, SettingBooleanDefault);
            }
            set
            {
                if (AddOrUpdateValue(SettingBooleanKeyName, value))
                {
                    Save();
                }
            }
        }

        const string SettingIntegerKeyName = "SettingInteger";
        const int SettingIntegerDefault = 5;
        public int SettingInteger
        {
            get
            {
                return GetValueOrDefault<int>(SettingIntegerKeyName, SettingIntegerDefault);
            }
            set
            {
                if (AddOrUpdateValue(SettingIntegerKeyName, value))
                {
                    Save();
                }
            }
        }

        SettingEnum SettingEnumDefault = SettingEnum.Default;
        const string SettingEnumKeyName = "SettingEnum";
        public SettingEnum SettingEnum
        {
            get
            {
                return GetValueOrDefault<SettingEnum>(SettingEnumKeyName, SettingEnumDefault);
            }
            set
            {
                if (AddOrUpdateValue(SettingEnumKeyName, value))
                {
                    Save();
                }
            }
        }
        public Collection<SettingEnum> AllSettingEnums
        {
            get
            {
                return SettingEnum.AllSettings;
            }
        }


#region Boilerplate code
        private static UserSettings _instance = null;
        public static UserSettings Instance { get { return _instance; } }
        public UserSettings()
        {
            try
            {
                settings = IsolatedStorageSettings.ApplicationSettings;
                if (_instance == null)
                {
                    _instance = this;
                }
                else
                {
                    System.Diagnostics.Debug.Assert(false, "Created multiple UserSettings!");
                }
            }
            catch (Exception)
            {
                settings = null;
            }
        }

        /// <summary>
        /// Update a setting value for our application. If the setting does not
        /// exist, then add the setting.
        /// </summary>
        /// <param name="Key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public bool AddOrUpdateValue(string Key, Object value)
        {
            bool valueChanged = false;

            // If the key exists
            if (settings.Contains(Key))
            {
                // If the value has changed
                if (settings[Key] != value)
                {
                    // Store the new value
                    settings[Key] = value;
                    valueChanged = true;
                }
            }
            // Otherwise create the key.
            else
            {
                settings.Add(Key, value);
                valueChanged = true;
            }
            return valueChanged;
        }

        /// <summary>
        /// Get the current value of the setting, or if it is not found, set the 
        /// setting to the default setting.
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="Key"></param>
        /// <param name="defaultValue"></param>
        /// <returns></returns>
        public T GetValueOrDefault<T>(string Key, T defaultValue)
        {
            T value;

            // If the key exists, retrieve the value.
            if (settings.Contains(Key))
            {
                value = (T)settings[Key];
            }
            // Otherwise, use the default value.
            else
            {
                value = defaultValue;
            }
            return value;
        }

        /// <summary>
        /// Save the settings.
        /// </summary>
        public void Save()
        {
            settings.Save();
            EventHandler settingsChanged = SettingsChanged;
            if (settingsChanged != null)
            {
                settingsChanged(this, new EventArgs());
            }
        }
#endregion
    }
}
